package util

import groovy.transform.CompileStatic

@CompileStatic
class ReleaseArtifactNames {

    static List<ArtifactNameResolver> artifactFileNameResolvers = new ArrayList<>()

    static Optional<String> getNewArtifactName(File file, String dockerfileName) {
        def newArtifactName = artifactFileNameResolvers.stream()
            .map { it.resolveArtifactNewName(file, dockerfileName) }
            .filter { it != null }
            .firstOrNull()

        return Optional.ofNullable(newArtifactName)
    }

    static void registerArtifactResolver(ArtifactNameResolver resolver) {
        artifactFileNameResolvers << resolver
    }

    static {
        registerArtifactResolver { file, dockerfile ->
            CIProperties.findProperty("release.artifact.${dockerfile}.${replacePathInvalidChars(file.absoluteCanonicalFile.path)}.name").orNull()
        }

        registerArtifactResolver { file, dockerfile ->
            CIProperties.findProperty("release.artifact.${dockerfile}.${file.name}.name").orNull()
        }

        registerArtifactResolver { file, dockerfile ->
            CIProperties.findProperty("release.artifact.${replacePathInvalidChars(file.absoluteCanonicalFile.path)}.name").orNull()
        }

        registerArtifactResolver { file, dockerfile ->
            CIProperties.findProperty("release.artifact.${file.name}.name").orNull()
        }

        // В рамках обратной совместимости
        registerArtifactResolver { file, dockerfile ->
            CIProperties.findProperty("gitea.release.artifact.${dockerfile}.${replacePathInvalidChars(file.absoluteCanonicalFile.path)}.name").orNull()
        }

        registerArtifactResolver { file, dockerfile ->
            CIProperties.findProperty("gitea.release.artifact.${dockerfile}.${file.name}.name").orNull()
        }

        registerArtifactResolver { file, dockerfile ->
            CIProperties.findProperty("gitea.release.artifact.${replacePathInvalidChars(file.absoluteCanonicalFile.path)}.name").orNull()
        }

        registerArtifactResolver { file, dockerfile ->
            CIProperties.findProperty("gitea.release.artifact.${file.name}.name").orNull()
        }
    }

    static String replacePathInvalidChars(String fileFullName) {
        return fileFullName
                .replace('\\', '_')
                .replace('/', '_')
    }

    static interface ArtifactNameResolver {
        String resolveArtifactNewName(File artifactFile, String dockerfileName)
    }
}