package util

import groovy.transform.CompileStatic

@CompileStatic
class ReleaseDescriptionReader {

    static Optional<String> readDescription(String releaseName) {
        def releaseDescriptionsDir = CIProperties.findProperty("release.descriptions.dir")
            .orElse(".releases/changelog/")
            .with {
                new File(it)
            }

        def releaseDescriptionFile = releaseDescriptionsDir.subFile("${releaseName}-changelog.md")

        if (releaseDescriptionFile.exists()) {
            def releaseAbsPath = releaseDescriptionFile.absoluteCanonicalFile.path
            ScriptLog.printf "Found description file for '$releaseName' release in '$releaseAbsPath'! It will be used for release creation."
            return Optional.of(releaseDescriptionFile.getText("UTF-8"))
        } else {
            ScriptLog.printf "Release description file not found in '${releaseDescriptionFile.absoluteCanonicalFile.path}', falling back to default value..."
        }

        return Optional.empty()
    }
}
